/*
 * file ControllableFolder.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A proxy for a controllable folder resource.
 * 
 * A controllable folder resource is a folder that can be placed under control.
 * When a controllable folder is placed under version control, a version of that folder
 * only captures the named mappings to version controlled members of that folder.
 * In order to version control the entire configuration rooted at a folder,
 * the folder must be placed under "baseline control".
 * 
 * @since 1.0
 */
public interface ControllableFolder extends ControllableResource, Folder {

    /**
     * Put this ControllableFolder under baseline control.
     * If no ControllableFolder exists at this location, create one under baseline control.
     * <p>
     * A configuration resource is associated with
     * this ControllableFolder, which allows versions of the configuration rooted at
     * this ControllableFolder to be created by checking out and checking in the 
     * configuration resource. </p>
     * <p>
     * A version of a configuration is called a "baseline". </p>
     * <p>
     * Postconditions:
     * <li>(create-configuration): A new configuration is created,
     *  whose {@link Configuration#ROOT_FOLDER} property identifies this ControllableFolder.
     *  The {@link ControllableResource#WORKSPACE} property of the new configuration is the same as 
     *  the {@link ControllableResource#WORKSPACE} property of the controllable folder identified by
     *  this ControllableFolder.
     * <li>(reference-configuration): The {@link ControllableResource#CONFIGURATION} property of the folder
     *  and all its members identifies the new configuration. The {@link #ROOT_FOLDER_OF}
     *  property of the folder identifies the new configuration.
     * <li>(create-new-baseline): The request MUST have created a new baseline history
     *  at a server-defined location, and MUST have created a new baseline in that baseline history.
     *  The {@link Baseline#VERSION_LIST} of the new baseline MUST identify a
     *  list of the {@link #CHECKED_IN} versions of each version-controlled member
     *  of the request folder.
     *  The {@link ControllableResource#CHECKED_IN} property of the new configuration MUST identify the new baseline.
     *  The {@link ControllableResource#VERSION_HISTORY} property of the new configuration MUST identify the new
     *  baseline history.
     *  
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#CONTROLLED_CONFIGURATION_ALREADY_EXISTS}
     *  If there already
     *  is a Configuration that identifies this ControllableFolder
     *  in its {@link Configuration#ROOT_FOLDER} property, the request MUST fail.
     * <li>{@link ReasonCode#NO_CHECKED_OUT_BASELINE_CONTROLLED_FOLDER_MEMBERS}:
     *  All version-controlled members of this folder MUST be checked-in.
     * <li>{@link ReasonCode#FORBIDDEN}:
     *  The folder must be baseline controllable.
     */
    public ControllableFolder doBaselineControl(Feedback feedback) throws WvcmException;

    /**
     * Create a baseline-controlled controllable folder at this location, and 
     * initialize it with the contents of the specified baseline.
     * If the specified baseline must be restored at a specific location,
     * the new baseline-controlled folder is created at that specific location
     * instead of this ControllableFolder.
     * The request will fail if a resource already exists at that location.
     * 
     * <p>
     * 
     * Postconditions:
     * <li>(create-configuration): A new baseline-controlled folder is created,
     *  and a new configuration is created whose {@link Configuration#ROOT_FOLDER} property
     *  identifies the new folder.
     * <li>(reference-configuration): The {@link ControllableResource#CONFIGURATION} property of the folder
     *  and all its members identifies the new configuration. The {@link #ROOT_FOLDER_OF}
     *  property of the folder identifies the new configuration.
     * <li>(select-existing-baseline): The {@link ControllableResource#CHECKED_IN} property of the new configuration
     *  MUST have been set to identify the specified baseline.
     *  A version-controlled member of the folder will be created for each version in the baseline,
     *  where the version-controlled member will have the content of that version,
     *  and will have the same name relative to the folder as the corresponding
     *  version-controlled resource had when the baseline was created.
     *  Any nested folders that are needed to provide the appropriate name
     *  for a version-controlled member will be created.
     *  
     * @param baseline the baseline used to initialize the folder.
     * @param feedback Specifies optional feedback to the caller.
     * @return A proxy for this new resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#CANNOT_CREATE_AT_THIS_LOCATION}:
     *  The location for the new baseline-controlled folder MUST be a valid location for a baseline-controlled folder.
     * <li>{@link ReasonCode#RESOURCE_ALREADY_EXISTS_AT_LOCATION}:
     *  The location for the new baseline-controlled folder MUST NOT identify an existing resource.
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     *  The baseline argument MUST identify a baseline.
     * <li>{@link ReasonCode#CANNOT_HAVE_MULTIPLE_BASELINE_CONTROLLED_FOLDERS}:
     *  There MUST NOT be another folder
     *  in the workspace of this ControllableFolder whose configuration property
     *  identifies a configuration for the baseline history of that baseline. 
     */
    public ControllableFolder doCreateBaselineControlledFolder(Baseline baseline, Feedback feedback) throws WvcmException;

    /**
     * True if the folder can be put under baseline control with the
     * {@link #doBaselineControl} or
     * {@link #doCreateBaselineControlledFolder} methods, otherwise false.
     */
    public static final PropertyName<Boolean> IS_BASELINE_CONTROLLABLE =
        new PropertyName<Boolean>("is-baseline-controllable"); //$NON-NLS-1$

    /**
     * Get the {@link #IS_BASELINE_CONTROLLABLE} property.
     * 
     * @return the {@link #IS_BASELINE_CONTROLLABLE} property.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #IS_BASELINE_CONTROLLABLE} as a wanted property.
     */
    public boolean getIsBaselineControllable() throws WvcmException;

    /**
     * The configuration for which this is the root folder.
     * The {@link #ROOT_FOLDER_OF} property is the computed inverse of the
     * {@link Configuration#ROOT_FOLDER} property.
     * @see #getRootFolderOf
     */
    public static final PropertyName<Configuration> ROOT_FOLDER_OF =
        new PropertyName<Configuration>("root-folder-of"); //$NON-NLS-1$

    /**
     * Get the {@link #ROOT_FOLDER_OF} property.
     * 
     * @return the {@link #ROOT_FOLDER_OF} property.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #ROOT_FOLDER_OF} as a wanted property.
     */
    public Configuration getRootFolderOf() throws WvcmException;

    /**
     * True if the folder has been baseline controlled. False if the folder
     * has not been baseline controlled or is a child of a folder that has
     * been baseline controlled.
     * @see #getIsBaselineControlled
     */
    public static final PropertyName<Boolean> IS_BASELINE_CONTROLLED =
        new PropertyName<Boolean>("is-baseline-controlled"); //$NON-NLS-1$

    /**
     * Get the {@link #IS_BASELINE_CONTROLLED} property.
     * 
     * @return the {@link #IS_BASELINE_CONTROLLED} property.
     * @throws WvcmException if this ControllableFolder was not created with
     * {@link #IS_BASELINE_CONTROLLED} as a wanted property.
     */
    public boolean getIsBaselineControlled() throws WvcmException;

}
